# Andrea Diamantini
# UDP GUI Client


import asyncio
import socket
import sys

import wx
import wxasync


class ChatWindow(wx.Frame):
    def __init__(self, name, clientAddress, serverAddress):
        super().__init__(None, title="Async UDP Client Chat")

        # member variables
        self.name = name
        self.clientAddress = clientAddress
        self.serverAddress = serverAddress

        # The NETWORK SOCKET async part --------
        self.udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.udp_socket.bind(clientAddress)
        self.udp_socket.setblocking(False)

        self.loop = asyncio.get_running_loop()

        self.udp_socket.sendto(("CONNECT " + self.name).encode(), self.serverAddress)

        # The GUI part -------------------------
        panel = wx.Panel(self)

        vbox = wx.BoxSizer(wx.VERTICAL)

        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        self.msgList = wx.ListBox(panel)
        hbox1.Add(self.msgList, proportion=1, flag=wx.ALL | wx.EXPAND, border=5)
        vbox.Add(hbox1, proportion=1, flag=wx.ALL | wx.EXPAND, border=5)

        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        self.tc1 = wx.TextCtrl(panel, style=wx.TE_PROCESS_ENTER)
        sendButton = wx.Button(panel, label="SEND")

        # AsyncBind (instead of usual Bind)
        wxasync.AsyncBind(wx.EVT_TEXT_ENTER, self.sendMessage, self.tc1)
        wxasync.AsyncBind(wx.EVT_BUTTON, self.sendMessage, sendButton)

        hbox2.Add(self.tc1, proportion=1, flag=wx.ALL, border=5)
        hbox2.Add(sendButton, proportion=0, flag=wx.ALL, border=5)
        vbox.Add(hbox2, proportion=0, flag=wx.ALL | wx.EXPAND, border=5)

        self.SetMinSize((200, 300))
        panel.SetSizer(vbox)
        self.Centre()

        # And now start receiving...
        wxasync.StartCoroutine(self.recvMessage, self)

    async def sendMessage(self, evt):
        msg = self.tc1.Value.strip()

        if msg:
            # Send it!
            self.udp_socket.sendto(msg.encode(), self.serverAddress)
            self.msgList.Append(f"[IO] {msg}")

        self.tc1.Clear()
        return

    async def recvMessage(self):
        while True:
            data = await self.loop.sock_recv(self.udp_socket, 1024)
            msg = data.decode()
            self.msgList.Append(msg)

        return


# ---------------------------------------------------------------------------------


async def runGuiClient(server_port):
    # THE App...
    app = wxasync.WxAsyncApp()

    diag1 = wx.TextEntryDialog(None, "Server host or IP: ")
    if diag1.ShowModal() != wx.ID_OK:
        sys.exit(0)
    server_host = diag1.GetValue()

    diag2 = wx.TextEntryDialog(None, "Set Name: ")
    if diag2.ShowModal() != wx.ID_OK:
        sys.exit(0)
    name = diag2.GetValue()

    serverAddress = (server_host, server_port)

    # client address
    import random

    client_host = socket.gethostname()
    client_ip = socket.gethostbyname(client_host)
    client_port = random.randint(22223, 55555)
    clientAddress = (client_ip, client_port)

    window = ChatWindow(name, clientAddress, serverAddress)
    window.Show()

    app.SetTopWindow(window)
    await app.MainLoop()


# ---------------------------------------------------------------------------------

if __name__ == "__main__":
    server_port = 22222

    try:
        asyncio.run(runGuiClient(server_port))

    # close on CTRL + C
    except KeyboardInterrupt:
        print("[INFO] Closing GUI client")



# Andrea Diamantini
# async UDP server

import asyncio
import socket


async def udpServer(server_port):
    serverAddress = ("0.0.0.0", server_port)

    clientList = {}

    loop = asyncio.get_running_loop()

    # UDP socket
    udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    udp_socket.bind(serverAddress)
    udp_socket.setblocking(False)

    print("UDP server up and listening")

    # Listen for incoming datagrams
    while True:
        data, address = await loop.sock_recvfrom(udp_socket, 1024)
        msg = data.decode()

        # connection protocol
        if msg.startswith("CONNECT"):
            wordList = msg.split()
            name = wordList[1]
            clientList[address] = name
            continue

        sender = clientList.get(address, "boh")
        name_msg = f"[{sender}] {msg}"
        print(name_msg)
        for addr in clientList:
            if addr != address:
                udp_socket.sendto(name_msg.encode(), addr)
                print(f"sending to {addr}")

    udp_socket.close()


async def runServer(server_port):
    # async gather execution
    await asyncio.gather(
        udpServer(server_port),
    )


if __name__ == "__main__":
    server_port = 22222

    try:
        # async run server
        asyncio.run(runServer(server_port))

    # close on CTRL + C
    except KeyboardInterrupt:
        print("[INFO] Server closed.")



# Andrea Diamantini
# UDP Client

import asyncio
import socket


async def sendMessage(udp_socket, serverAddress):
    loop = asyncio.get_running_loop()

    print(f"[INFO] Sending messages to {serverAddress}")
    print("[INFO] Write a message and press ENTER (/quit to exit)")
    print("-" * 50)

    while True:
        # async read input
        msg = await loop.run_in_executor(None, input)
        msg = msg.strip()

        if msg.lower() == "/quit":
            print("[INFO] Closing client... now press CTRL + C")
            break

        if msg:
            # sending msg
            udp_socket.sendto(msg.encode(), serverAddress)
            print(f"[ME] {msg}")

    udp_socket.close()


async def recvMessage(udp_socket):
    loop = asyncio.get_running_loop()

    print(f"[INFO] listening on {udp_socket.getsockname()}")

    while True:
        data = await loop.sock_recv(udp_socket, 1024)
        msg = data.decode()
        print(msg)

    udp_socket.close()


async def runClient(server_port):
    server_host = input("Server host or IP: ")
    name = input("Nome: ")

    serverAddress = (server_host, server_port)

    import random

    client_host = socket.gethostname()
    client_ip = socket.gethostbyname(client_host)
    client_port = random.randint(22223, 55555)
    clientAddress = (client_ip, client_port)

    # UDP socket
    udp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    udp_socket.bind(clientAddress)
    udp_socket.setblocking(False)

    udp_socket.sendto(("CONNECT " + name).encode(), serverAddress)

    # async gather execution
    await asyncio.gather(
        sendMessage(udp_socket, serverAddress),
        recvMessage(udp_socket),
    )


if __name__ == "__main__":
    server_port = 22222

    try:
        # async run process
        asyncio.run(runClient(server_port))

    # close on CTRL + C
    except KeyboardInterrupt:
        print("[INFO] Closing client...")



# Andrea Diamantini
# UDP async CHAT

import argparse
import asyncio
import sys
from importlib.metadata import version

# local import
from async_udp_chat import udp_client, udp_server

# server
server_port = 22222

# ---------------------------------------------------------------------------------------------------------


def main():
    """
    async udp chat, server and client app.
    Implemented for educational purposes.
    """

    parser = argparse.ArgumentParser(
        prog="async_udp_chat",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=f"{main.__doc__}",
    )

    parser.add_argument(
        "-v",
        "--version",
        help="show package version and exit",
        action="version",
        version=version("async_udp_chat"),
    )

    parser.add_argument("--server", help="run in server mode", action="store_true")
    parser.add_argument("--client", help="run in client mode", action="store_true")
    parser.add_argument("--gui", help="run in GUI client mode", action="store_true")

    args = parser.parse_args()

    try:
        if args.server:
            print("[INFO] server mode")
            asyncio.run(udp_server.runServer(server_port))
            sys.exit(0)

        if args.client:
            print("[INFO] client mode")
            asyncio.run(udp_client.runClient(server_port))
            sys.exit(0)

        if args.gui:
            print("[INFO] GUI client mode")
            # included here cause of the OPTIONAL wx deps
            from async_udp_chat import udp_gui_client

            asyncio.run(udp_gui_client.runGuiClient(server_port))
            sys.exit(0)

        # if no arguments, print help
        parser.print_help()

    except KeyboardInterrupt:
        print("[INFO] Closing NOW!!!")


# ---------------------------------------------------------------------------------------------------------

if __name__ == "__main__":
    main()
